
#include <windows.h>  
#include "DupHandl.h" 


HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName    = "MyApp";
LPCTSTR lpszTitle      = "My Application"; 
LPCTSTR lpszChildTitle = "Child Process";


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG        msg;
   HWND       hWnd; 
   WNDCLASSEX wc;

   // Register the main application window class.
   //............................................
   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon( hInstance, lpszAppName ); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              
   wc.cbSize        = sizeof(WNDCLASSEX);
   wc.hIconSm       = LoadImage( hInstance, lpszAppName, 
                                 IMAGE_ICON, 16, 16,
                                 LR_DEFAULTCOLOR );

   if ( !RegisterClassEx( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   // Create the main application window.
   //....................................
   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
   switch( uMsg )  
   {   
      case WM_CREATE:
               // If child set child name.
               //.........................
               if ( strstr( GetCommandLine(), "CHILD" ) )
                  SetWindowText( hWnd, lpszChildTitle );
               else 
                  // position parent to left of child
                  //.................................
                  MoveWindow( hWnd, 0, 0, 200, 200, TRUE );
               break;

      case WM_USER: 
              {
                 HDC hDC = GetDC( hWnd );
                 TextOut( hDC, 0, 0, "Waiting for mutex", 17);

                 // Get mutex.
                 //...........
                 WaitForSingleObject( ( HANDLE )wParam, INFINITE ); 
                 TextOut( hDC, 0, 20, "Got mutex", 9);

                 // Wait for a while.
                 //..................
                 Sleep( 3000 );

                 TextOut( hDC, 0, 40, "Released mutex", 14);
                 ReleaseMutex( ( HANDLE )wParam );
              }
              break;

      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_TEST: 
                        { 
                           HANDLE hAlias;
                           STARTUPINFO si;
                           PROCESS_INFORMATION pi;

                           // Create a process and export mutex handle.
                           //..........................................
                           ZeroMemory( &si, sizeof( STARTUPINFO ) );
                           ZeroMemory( &pi, sizeof( PROCESS_INFORMATION ) );

                           si.cb          = sizeof( STARTUPINFO );
                           si.dwFlags     = STARTF_USESHOWWINDOW;
                           si.wShowWindow = SW_HIDE;

                           CreateProcess( NULL, "DUPHANDL.EXE CHILD", 
                                          NULL, NULL, FALSE, 
                                          0, NULL, NULL, &si, &pi );

                           // If process is created.
                           //.......................
                           if ( pi.hProcess ) 
                           { 
                              HWND hWndChild;
                              HANDLE hMutex = CreateMutex( NULL, FALSE, "TEST_MUTEX" );

                              DuplicateHandle( GetCurrentProcess(), hMutex,
                                               pi.hProcess, &hAlias, 
                                               SYNCHRONIZE, FALSE, 0 );

                              // Let initialization of new process occur.
                              //.........................................
                              WaitForInputIdle( GetCurrentProcess(), INFINITE );

                              // Get external window address.
                              //.............................
                              while ( (hWndChild = FindWindow( lpszAppName, lpszChildTitle )) == NULL);

                              // Move it over to left.
                              //......................
                              MoveWindow( hWndChild, 200, 0, 200, 200, TRUE );
                              ShowWindow( hWndChild, SW_SHOWNORMAL );

                              PostMessage( hWndChild, WM_USER, ( WPARAM )hAlias, 0 );
                              CloseHandle( hMutex );

                              // Wait for a while.
                              //..................
                              Sleep( 5000 );

                              // Terminate the child process.
                              //.............................
                              TerminateProcess( pi.hProcess, 0 );
                           }
                        }
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
