
#include <windows.h>  
#include "MsgWait.h" 


HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName  = "MyApp";
LPCTSTR lpszTitle    = "My Application"; 


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG        msg;
   HWND       hWnd; 
   WNDCLASSEX wc;

   // Register the main application window class.
   //............................................
   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon( hInstance, lpszAppName ); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              
   wc.cbSize        = sizeof(WNDCLASSEX);
   wc.hIconSm       = LoadImage( hInstance, lpszAppName, 
                                 IMAGE_ICON, 16, 16,
                                 LR_DEFAULTCOLOR );

   if ( !RegisterClassEx( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   // Create the main application window.
   //....................................
   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}

LPCTSTR lpszPulseEvent = "PULSE_EVENT";

DWORD WINAPI ChildThreadProc( LPDWORD lpdwData )
{
   HWND   hThreadWnd;      // Handle of window for this thread.
   HANDLE hEvent;          // Handle to event.
   TCHAR  szBuffer[128];   // General work area for string formatting.
   HWND   hList  = (HWND)lpdwData;

   wsprintf( szBuffer, "Thread %lX", GetCurrentThreadId() );
   hThreadWnd = CreateWindow( lpszAppName, szBuffer, WS_OVERLAPPEDWINDOW, 0, 0,
                              100, 100, (HWND)NULL, NULL, hInst, (LPTSTR)NULL );
   if (!hThreadWnd)
      return( 1 );

   // Display the window and open the pulse event.
   //.............................................
   ShowWindow( hThreadWnd, SW_SHOWNORMAL );
   UpdateWindow( hThreadWnd );

   hEvent = OpenEvent( SYNCHRONIZE, FALSE, lpszPulseEvent );

   // Loop forever going into and out of wait state.
   //...............................................
   while( TRUE )  
   {
      MSG   msg;    
      DWORD dwTest; 

      // Use a PeekMessage() loop so it will NOT wait for
      // the next message to be placed in the queue.
      //.................................................
      while( PeekMessage( &msg, 0, 0, 0, PM_REMOVE )  )
      {
         TranslateMessage( &msg );
         DispatchMessage( &msg );
      }

      wsprintf( szBuffer, "Thread %x waiting for Event %x or Keyboard input.",
                GetCurrentThreadId(), hEvent );

      SendMessage( hList, LB_INSERTSTRING, (WPARAM)-1, (LPARAM)szBuffer );

      // Check for manual event or keyboard input.
      //..........................................
      dwTest = MsgWaitForMultipleObjects( 1, &hEvent, FALSE, 10000, QS_KEY );
      if (dwTest != WAIT_TIMEOUT)
         wsprintf( szBuffer,"Thread %x satisfied condition.", GetCurrentThreadId() );
      else
         wsprintf( szBuffer,"Thread %x timed out.", GetCurrentThreadId() );

      SendMessage( hList, LB_INSERTSTRING, (WPARAM)-1, (LPARAM)szBuffer );
   }

   CloseHandle( hEvent );

   return( 0 );
}


LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static HWND   hList = NULL;
static HANDLE hEvent = NULL;

   switch( uMsg )  
   {   
      case WM_CREATE:
              // Create list box.
              //.................
              if ( !hList )
                 hList  = CreateWindowEx( WS_EX_CLIENTEDGE, "LISTBOX", "", 
                                          LBS_STANDARD | LBS_NOINTEGRALHEIGHT | 
                                          WS_CHILD | WS_VISIBLE,
                                          0, 0, 10, 10,
                                          hWnd, (HMENU)101,
                                          hInst, NULL );

              if ( !hEvent )
              {
                 SECURITY_ATTRIBUTES sa;

                 sa.nLength              = sizeof( SECURITY_ATTRIBUTES );
                 sa.lpSecurityDescriptor = NULL; 
                 sa.bInheritHandle       = TRUE;
                 hEvent = CreateEvent( &sa, TRUE, FALSE, lpszPulseEvent );
              }
              break;

      case WM_SIZE :
              if ( wParam != SIZE_MINIMIZED && hWnd == GetParent( hList ) )
                 MoveWindow( hList, 0, 0, LOWORD( lParam ), HIWORD( lParam ), TRUE );
              break;

      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_TEST:
                        {
                           DWORD dwChildId;

                           CreateThread( NULL, 0, ChildThreadProc, hList, 0, &dwChildId );
                        }
                        break;

                 case IDM_PULSE:
                        SendMessage( hList, LB_INSERTSTRING, (WPARAM)-1, (LPARAM)"Pulsing Event" );
                        PulseEvent( hEvent );
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              if ( hEvent )
              {
                 CloseHandle( hEvent );
                 hEvent = NULL;
              }

              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
