
#include <windows.h>  
#include "OpenProc.h" 


HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName  = "MyApp";
LPCTSTR lpszTitle    = "My Application"; 


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG        msg;
   HWND       hWnd; 
   WNDCLASSEX wc;

   // Register the main application window class.
   //............................................
   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon( hInstance, lpszAppName ); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              
   wc.cbSize        = sizeof(WNDCLASSEX);
   wc.hIconSm       = LoadImage( hInstance, lpszAppName, 
                                 IMAGE_ICON, 16, 16,
                                 LR_DEFAULTCOLOR );

   if ( !RegisterClassEx( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   // Create the main application window.
   //....................................
   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


LPVOID lpHeapData          = NULL;
LPVOID lpExtData           = NULL;
DWORD  dwDebuggeeProcessID = 0;
TCHAR  szBuffer[256];


LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
   switch( uMsg )  
   {   
      case WM_CREATE:
               {
                  // Allocate some heap data for exchange.
                  //......................................
                  lpHeapData = HeapAlloc( GetProcessHeap(), 0, 256 );

                  // If child set child name.
                  //.........................
                  if ( strstr( GetCommandLine(), "DEBUGGEE" ) )
                  {
                     SetWindowText( hWnd, "DEBUGGEE" );
                     dwDebuggeeProcessID = GetCurrentProcessId();
                     lstrcpy( lpHeapData, "@@@ Debuggee Data @@@");
                     SetMenu( hWnd, 0 );  
                  }
                  else
                  { 
                     // Position parent above child.
                     //.............................
                     SetWindowText( hWnd, "DEBUGGER" );
                     lstrcpy(lpHeapData, "<<<< Debugger Data >>>>");
                  }
               }
               break;

      case WM_USER: 
               // Used to return value of the heap data.
               //.......................................
               return( (LRESULT)lpHeapData );

      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                  case IDM_LAUNCH:
                        {
                           STARTUPINFO si;
                           PROCESS_INFORMATION pi;
                           HWND hWndEx;

                           // Only one can be debugged at a time.
                           //....................................
                           if (FindWindow(lpszAppName, "DEBUGGEE")!=0)
                              break;

                           ZeroMemory(&si, sizeof(STARTUPINFO));
                           ZeroMemory(&pi, sizeof(PROCESS_INFORMATION));
                           si.cb=sizeof(STARTUPINFO);
                           CreateProcess( NULL, "OPENPROC.EXE DEBUGGEE", 
                                          NULL, NULL, FALSE, 0, NULL,
                                          NULL, &si, &pi );
                           dwDebuggeeProcessID = pi.dwProcessId;

                           // Let initialization of new process occur.
                           //.........................................
                           if ( pi.dwProcessId )
                           {
                              WaitForInputIdle( GetCurrentProcess(), INFINITE );
                              while ((hWndEx = FindWindow(lpszAppName, "DEBUGGEE"))==0);
                              lpExtData = (LPVOID) SendMessage(hWndEx, WM_USER, 0, 0);
                           }
                        }
                        break;

                 case IDM_READ:
                        {
                           TCHAR szData[256];
                           DWORD dwBytesRead = 0;
                           HANDLE hDebuggee = OpenProcess( PROCESS_VM_READ, FALSE,
                                                           dwDebuggeeProcessID );

                           ReadProcessMemory( hDebuggee, lpExtData,
                                              szData, 255, &dwBytesRead );
                           wsprintf( szBuffer, "%ld Bytes Read. Data=%s",
                                     dwBytesRead, szData );
                           MessageBox( hWnd, szBuffer, "Reading Process", MB_OK );

                           CloseHandle( hDebuggee );
                        }
                        break;

                 case IDM_WRITE:
                        {
                           DWORD dwBytesWritten = 0;
                           HANDLE hDebuggee = OpenProcess( PROCESS_VM_WRITE, FALSE,
                                                           dwDebuggeeProcessID );

                           WriteProcessMemory( hDebuggee, lpExtData,
                                               lpHeapData, 255, &dwBytesWritten );
                           wsprintf(szBuffer, "%ld Bytes Written", dwBytesWritten);
                           MessageBox( hWnd, lpExtData, "Writing Process", MB_OK );

                           CloseHandle( hDebuggee );
                        }
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              HeapFree( GetProcessHeap(), 0, lpHeapData );
              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
